/*
 * Copyright (c) 2007, team-naver.com
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
 * NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
 * TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */
package com.eromedayo.poem.servlet.command;

import java.io.*;
import java.util.*;
import javax.servlet.http.*;
import com.eromedayo.poem.*;

public class UpdatePoemCommand extends Command {
	public UpdatePoemCommand() {}

	public String getName() {
		return "UpdatePoem";
	}

	public void exec(PoemServer base, PrintWriter out, HttpServletRequest request, HttpServletResponse response) throws Exception {
		HttpSession session = request.getSession(true);
		User user = (User)session.getAttribute("user");

		if(user == null) {
			user = User.getGuest();
			session.setAttribute("user", user);
		}

		if(!user.isEditor()) {
			out.println("OCĂ܂B");
			return;
		}

		Storage db = null;
		
		try {
			int poemId = PoemServer.toNum(request.getParameter("PoemID"));
			String profile = request.getParameter("Profile");

			Vector<Integer> tagIds = new Vector<Integer>();

			Enumeration params = request.getParameterNames();

			while(params.hasMoreElements()) {
				String paramName = (String)params.nextElement();
				if(!paramName.startsWith("tag_")) continue;
				if(!"on".equals(request.getParameter(paramName))) continue;

				String sTagId = paramName.substring("tag_".length());

				tagIds.addElement(PoemServer.toNum(sTagId));
			}

			db = new Storage();
			
			db.updatePoem(poemId, profile, tagIds);

			outRedirectScript(out, request);
		} catch(Exception e) {
			e.printStackTrace(out);
		} finally {
			if(db != null) db.close();
		}
	}
}
